/*
 * This program simulates particles moving about in a confined space.
 *
 * Authors:
 *     Aaron Weeden, 2013
 *     Phil List, 2013
 *     Aaron Weeden, 2014 (added randomColor and flipBG functions)
 *	   Anja Sheppard, 2016
 */

/*
 * Shodor Apprentices! Your assignment: add some code to this ouncing particles program to have the colors change like a rainbow. Hint: colors in a computer are RGB (red green blue).
 */

// Global variables
var BG_COLOR = "white";
var ParticleWidth = 10;     // the number of pixels wide each particle is.
var ParticleHeight = 10;    // the number of pixels tall each particle is.
var NumParticles = 50;      // the number of particles to use.
var IsBounceEnabled = true;    // says whether particles should bounce off each
                            //   other.
var ParticleColor = "getRandomColor()"; // the CSS color of the particles.
var IsOneHighlighted = false;  // says whether one particle should be
                               // highlighted.
var HighlightColor = "red"; // the CSS color of the particle being highlighted.
var Interval = 20;          // the number of milliseconds in between frames of
                            //   animation.
var Particles = [];         // an array holding the particles.
var Canvas;                 // holds the HTML canvas.
var Context;                // holds the drawing context of the HTML canvas.

// This function initializes the particles and calls other needed functions.
onload = function () {
  var i;
  var j;
  var randomDir;

  // Store the HTML canvas and its context in variables.
  Canvas = document.getElementById("canvas");
  Context = Canvas.getContext("2d");

  // Generate random starting locations and directions for the particles.
  for (i = 0; i < NumParticles; i++) {
    // Each particle is an object.
    Particles[i] = {};

    // Each particle object has an (x, y) position and a color.
    Particles[i].x =
      Math.floor(Math.random() * (Canvas.width - ParticleWidth));
    Particles[i].y =
      Math.floor(Math.random() * (Canvas.height - ParticleHeight));
    Particles[i].color = getRandomColor();

    // Generate a random number between 0 and 4, which will represent the
    // direction of the particle.
    randomDir = Math.floor(Math.random() * 4);

    // Based on the random number, decide whether the particle should be
    // moving to the top-left, top-right, bottom-left, or bottom-right.
    switch (randomDir) {
      case 0:
        Particles[i].xDirection = "left";
        Particles[i].yDirection = "up";
        break;
      case 1:
        Particles[i].xDirection = "right";
        Particles[i].yDirection = "up";
        break;
      case 2:
        Particles[i].xDirection = "left";
        Particles[i].yDirection = "down";
        break;
      case 3:
        Particles[i].xDirection = "right";
        Particles[i].yDirection = "down";
        break;
    }
  }

  // If we are highlighting a particle (e.g. particle 0), set that color's
  // particle.
  if (IsOneHighlighted) {
    Particles[0].color = HighlightColor;
  }

  // Start the animation by calling the animate function every Interval.
  setInterval("animate();", Interval);
}

// This function determines whether or not two particles have intersected.
// If particles should not be bouncing, this function returns false.
intersects = function (particle1, particle2) {
  if (IsBounceEnabled) {
    return (particle1.x < particle2.x + ParticleWidth &&
            particle2.x < particle1.x + ParticleWidth &&
            particle1.y < particle2.y + ParticleHeight &&
            particle2.y < particle1.y + ParticleHeight);
  }
  else {
    return false;
  }
}

// This function computes the direction each particle will travel.
// Then the drawing function is called.
animate = function () {
  var i;
  var j;

  // Loop over each pair of particles that intersect.
  for (i = 0; i < NumParticles; i++) {
    for (j = 0; j < NumParticles; j++) {
      if (i !== j) {
        if (intersects(Particles[i], Particles[j])) {
          // Decide the new x direction of the particles
          if (Particles[i].x < Particles[j].x) {
            Particles[i].xDirection = "left";
            Particles[j].xDirection = "right";
          }

          // Decide the new y direction of the particles
          if (Particles[i].y < Particles[j].y) {
            Particles[i].yDirection = "up";
            Particles[j].yDirection = "down";
          }
        }
      }
    }

    // Bounce particles off the walls
    // left
    if ((Particles[i].xDirection === "left") && (Particles[i].x === 0)) {
      Particles[i].xDirection = "right";
    }
    // right
    else if ((Particles[i].xDirection === "right") &&
        (Particles[i].x === Canvas.width - ParticleWidth)) {
      Particles[i].xDirection = "left";
    }
    // top
    if ((Particles[i].yDirection === "up") && (Particles[i].y === 0)) {
      Particles[i].yDirection = "down";
    }
    // bottom
    else if ((Particles[i].yDirection === "down") &&
        (Particles[i].y === Canvas.height - ParticleHeight)) {
      Particles[i].yDirection = "up";
    }

    // Move particles to new positions
    if ((Particles[i].xDirection === "left")) {
      Particles[i].x--;
    }
    else if (Particles[i].xDirection === "right") {
      Particles[i].x++;
    }
    if (Particles[i].yDirection === "up") {
      Particles[i].y--;
    }
    else if (Particles[i].yDirection == "down") {
      Particles[i].y++;
    }
  }

  // Draw the particles on the canvas.
  draw();
}
var red = 255;
var green = 0;
var blue = 0;
/*var red = (Math.floor(Math.random()*100));
var green = (Math.floor(Math.random()*100));
var blue = (Math.floor(Math.random()*100));*/
// This function clears the canvas and draws the particles on the screen.
draw = function () {
  var i;

  // Clear the canvas.
  Context.fillStyle = BG_COLOR;
  Context.fillRect(0, 0, Canvas.width, Canvas.height);

  // Draw each particle.
  for (i = 0; i < NumParticles; i++) {
    Context.fillStyle = "rgb(" + red + ", " + green + ", " + blue + ")"/*Particles[i].color*/;
    Context.fillRect(Particles[i].x, Particles[i].y, ParticleWidth,
      ParticleHeight);
  }
}

getRandomColor = function () {
	var red = Math.floor(Math.random() * 255);
	var green = Math.floor(Math.random() * 255);
	var blue = Math.floor(Math.random() * 255);

	return "rgb(" + red + "," + green + "," + blue + ")";
}

flipBG = function () {
	BG_COLOR = (BG_COLOR == "white") ? "black" : "white";
}
